@@ -10,18 +10,23 @@ import os
 
 try:
     os.environ.pop("LIBROSA_CACHE_DIR")
-except:
+except KeyError:
     pass
 
+import sys
 import soundfile
 import audioread.rawread
 import librosa
+import librosa.core
+import librosa.core.spectrum
 import glob
 import numpy as np
 import scipy.io
+import scipy.signal
 import pytest
 import warnings
 from unittest import mock
+from typing import Any, Callable, Union, cast
 
 
 # -- utilities --#
@@ -70,9 +75,10 @@ def test_load_soundfile():
     assert np.isclose(sr, sr2)
 
 
+@pytest.mark.filterwarnings("ignore:librosa.core.audio.__audioread_load")
 def test_load_audioread():
     fname = os.path.join("tests", "data", "test1_44100.wav")
-    
+
     # Load using an existing audioread object
     reader = audioread.rawread.RawAudioFile(fname)
     y, sr = librosa.load(reader, sr=None)
@@ -84,11 +90,11 @@ def test_load_audioread():
     assert np.isclose(sr, sr2)
 
 
-@pytest.mark.parametrize("res_type", ["kaiser_fast", "kaiser_best", "scipy"])
+@pytest.mark.parametrize("res_type", ["soxr_qq", "soxr_hq", "scipy"])
 def test_load_resample(res_type):
 
     sr_target = 16000
-    fn = librosa.ex("trumpet")
+    fn = os.path.join("tests", "data", "test1_44100.wav")
 
     y_native, sr = librosa.load(fn, sr=None, res_type=res_type)
 
@@ -169,10 +175,12 @@ def test_resample_mono(resample_mono, sr_out, res_type, fix):
     y, sr_in = resample_mono
     y = librosa.to_mono(y)
 
-    y2 = librosa.resample(y, orig_sr=sr_in, target_sr=sr_out, res_type=res_type, fix=fix)
+    y2 = librosa.resample(
+        y, orig_sr=sr_in, target_sr=sr_out, res_type=res_type, fix=fix
+    )
 
     # First, check that the audio is valid
-    librosa.util.valid_audio(y2, mono=True)
+    librosa.util.valid_audio(y2)
 
     # If it's a no-op, make sure the signal is untouched
     if sr_out == sr_in:
@@ -213,10 +221,12 @@ def test_resample_stereo(resample_audio, sr_out, res_type, fix):
 
     y, sr_in = resample_audio
 
-    y2 = librosa.resample(y, orig_sr=sr_in, target_sr=sr_out, res_type=res_type, fix=fix)
+    y2 = librosa.resample(
+        y, orig_sr=sr_in, target_sr=sr_out, res_type=res_type, fix=fix
+    )
 
     # First, check that the audio is valid
-    librosa.util.valid_audio(y2, mono=False)
+    librosa.util.valid_audio(y2)
 
     assert y2.ndim == y.ndim
 
@@ -250,10 +260,12 @@ def test_resample_scale(resample_mono, res_type, sr_out):
 
     y, sr_in = resample_mono
 
-    y2 = librosa.resample(y, orig_sr=sr_in, target_sr=sr_out, res_type=res_type, scale=True)
+    y2 = librosa.resample(
+        y, orig_sr=sr_in, target_sr=sr_out, res_type=res_type, scale=True
+    )
 
     # First, check that the audio is valid
-    librosa.util.valid_audio(y2, mono=True)
+    librosa.util.valid_audio(y2)
 
     n_orig = np.sqrt(np.sum(np.abs(y) ** 2))
     n_res = np.sqrt(np.sum(np.abs(y2) ** 2))
@@ -281,6 +293,7 @@ def test_stft(infile):
         os.path.join("tests", DATA["wavfile"][0]), sr=None, mono=True
     )
 
+    window: Union[Callable[..., Any], str]
     if DATA["hann_w"][0, 0] == 0:
         # Set window to ones, swap back to nfft
         window = np.ones
@@ -321,22 +334,97 @@ def test_stft_winsizes():
     x = np.zeros(1000000)
 
     for power in range(12, 17):
-        N = 2 ** power
+        N = 2**power
         H = N // 2
         librosa.stft(x, n_fft=N, hop_length=H, win_length=N)
 
 
+@pytest.mark.parametrize("center", [False, True])
+@pytest.mark.parametrize(
+    "n_fft, hop_length",
+    [(1023, 128), (1023, 129), (1023, 256), (2048, 512), (2048, 2048)],
+)
+@pytest.mark.parametrize("N", [1024, 2048, 8192])
+def test_stft_preallocate(center, n_fft, hop_length, N):
+
+    # Work in stereo by default
+    y = np.random.randn(2, max(N, n_fft))
+
+    D1 = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length)
+    out = np.empty_like(D1)
+    D2 = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length, out=out)
+    assert D2 is out
+    assert np.allclose(D1, D2)
+
+
+@pytest.mark.parametrize("center", [False, True])
+@pytest.mark.parametrize(
+    "n_fft, hop_length",
+    [(1023, 128), (1023, 129), (1023, 256), (2048, 512), (2048, 2048)],
+)
+@pytest.mark.parametrize("N", [2048])
+def test_stft_preallocate_oversize(center, n_fft, hop_length, N):
+
+    # Work in stereo by default
+    y = np.random.randn(2, max(N, n_fft))
+
+    D1 = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length)
+    shape = list(D1.shape)
+    shape[-1] *= 2
+    out = np.empty_like(D1, shape=shape)
+    D2 = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length, out=out)
+    assert np.allclose(D1, D2)
+    assert np.allclose(D1, out[..., : D2.shape[-1]])
+
+
+@pytest.mark.parametrize("center", [False, True])
+@pytest.mark.parametrize(
+    "n_fft, hop_length",
+    [(1023, 128), (1023, 129), (1023, 256), (2048, 512), (2048, 2048)],
+)
+@pytest.mark.parametrize("N", [2048])
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_stft_preallocate_undersize(center, n_fft, hop_length, N):
+
+    # Work in stereo by default
+    y = np.random.randn(2, max(N, n_fft))
+
+    D1 = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length)
+    shape = list(D1.shape)
+    shape[-1] //= 2
+    out = np.empty_like(D1, shape=shape)
+    D2 = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length, out=out)
+
+
+@pytest.mark.parametrize("center", [False, True])
+@pytest.mark.parametrize(
+    "n_fft, hop_length",
+    [(1023, 128), (1023, 129), (1023, 256), (2048, 512), (2048, 2048)],
+)
+@pytest.mark.parametrize("N", [1024, 2048, 8192])
+def test_istft_preallocate(center, n_fft, hop_length, N):
+    y = np.random.randn(2, max(N, n_fft))
+
+    D = librosa.stft(y, center=center, n_fft=n_fft, hop_length=hop_length)
+
+    y1 = librosa.istft(D, center=center, n_fft=n_fft, hop_length=hop_length)
+    y2 = np.empty_like(y1)
+    y3 = librosa.istft(D, center=center, n_fft=n_fft, hop_length=hop_length, out=y2)
+
+    assert y3 is y2
+    assert np.allclose(y1, y2)
+
+
 # results for FFT bins containing multiple components will be unstable, as when
 # using higher sampling rates or shorter windows with this test signal
-@pytest.mark.parametrize("center", [False, True])
+@pytest.mark.parametrize("center", [False])
 @pytest.mark.parametrize("sr", [256, 512, 2000, 2048])
 @pytest.mark.parametrize("n_fft", [128, 255, 256, 512, 1280])
 def test___reassign_frequencies(sr, n_fft, center):
     x = np.linspace(0, 5, 5 * sr, endpoint=False)
     y = np.sin(17 * x * 2 * np.pi) + np.sin(103 * x * 2 * np.pi)
-
     freqs, S = librosa.core.spectrum.__reassign_frequencies(
-        y=y, sr=sr, n_fft=n_fft, hop_length=n_fft, center=center, pad_mode="wrap"
+        y=y, sr=sr, n_fft=n_fft, hop_length=n_fft, center=center
     )
 
     S_db = librosa.amplitude_to_db(np.abs(S), ref=np.max)
@@ -395,11 +483,9 @@ def test___reassign_times(sr, n_fft):
     expected_times = librosa.samples_to_time(impulse_indices, sr=sr)
     expected[:, expected_bins] = np.tile(expected_times, (n_fft // 2 + 1, 1))
 
-    # ignore divide-by-zero warnings for frames with no energy
-    with warnings.catch_warnings(record=True):
-        times, S = librosa.core.spectrum.__reassign_times(
-            y=y, sr=sr, n_fft=n_fft, hop_length=n_fft, center=False
-        )
+    times, S = librosa.core.spectrum.__reassign_times(
+        y=y, sr=sr, n_fft=n_fft, hop_length=n_fft, center=False
+    )
 
     # times should be reassigned within 0.5% of the window duration
     assert np.allclose(times, expected, atol=0.005 * n_fft / sr, equal_nan=True)
@@ -412,11 +498,9 @@ def test___reassign_times_center():
     sr = 4000
     n_fft = 2048
 
-    # ignore divide-by-zero warnings for frames with no energy
-    with warnings.catch_warnings(record=True):
-        times, S = librosa.core.spectrum.__reassign_times(
-            y=y, sr=sr, hop_length=n_fft, win_length=n_fft, center=True
-        )
+    times, S = librosa.core.spectrum.__reassign_times(
+        y=y, sr=sr, hop_length=n_fft, win_length=n_fft, center=True
+    )
 
     expected = np.full_like(times, np.nan)
     expected[:, 1] = 2049 / float(sr)
@@ -605,7 +689,12 @@ def test_salience_basecase():
     harms = [1]
     weights = [1.0]
     S_sal = librosa.core.salience(
-        S, freqs=freqs, harmonics=harms, weights=weights, filter_peaks=False, kind="quadratic"
+        S,
+        freqs=freqs,
+        harmonics=harms,
+        weights=weights,
+        filter_peaks=False,
+        kind="quadratic",
     )
     assert np.allclose(S_sal, S)
 
@@ -617,7 +706,12 @@ def test_salience_basecase2():
     harms = [1, 0.5, 2.0]
     weights = [1.0, 0.0, 0.0]
     S_sal = librosa.core.salience(
-        S, freqs=freqs, harmonics=harms, weights=weights, filter_peaks=False, kind="quadratic"
+        S,
+        freqs=freqs,
+        harmonics=harms,
+        weights=weights,
+        filter_peaks=False,
+        kind="quadratic",
     )
     assert np.allclose(S_sal, S)
 
@@ -626,7 +720,9 @@ def test_salience_defaults():
     S = np.array([[0.1, 0.5, 0.0], [0.2, 1.2, 1.2], [0.0, 0.7, 0.3], [1.3, 3.2, 0.8]])
     freqs = np.array([50.0, 100.0, 200.0, 400.0])
     harms = [0.5, 1, 2]
-    actual = librosa.core.salience(S, freqs=freqs, harmonics=harms, kind="quadratic", fill_value=0.0)
+    actual = librosa.core.salience(
+        S, freqs=freqs, harmonics=harms, kind="quadratic", fill_value=0.0
+    )
 
     expected = (
         np.array([[0.0, 0.0, 0.0], [0.3, 2.4, 1.5], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]])
@@ -641,7 +737,12 @@ def test_salience_weights():
     harms = [0.5, 1, 2]
     weights = [1.0, 1.0, 1.0]
     actual = librosa.core.salience(
-        S, freqs=freqs, harmonics=harms, weights=weights, kind="quadratic", fill_value=0.0
+        S,
+        freqs=freqs,
+        harmonics=harms,
+        weights=weights,
+        kind="quadratic",
+        fill_value=0.0,
     )
 
     expected = (
@@ -657,7 +758,12 @@ def test_salience_no_peak_filter():
     harms = [0.5, 1, 2]
     weights = [1.0, 1.0, 1.0]
     actual = librosa.core.salience(
-        S, freqs=freqs, harmonics=harms, weights=weights, filter_peaks=False, kind="quadratic"
+        S,
+        freqs=freqs,
+        harmonics=harms,
+        weights=weights,
+        filter_peaks=False,
+        kind="quadratic",
     )
 
     expected = (
@@ -673,7 +779,13 @@ def test_salience_aggregate():
     harms = [0.5, 1, 2]
     weights = [1.0, 1.0, 1.0]
     actual = librosa.core.salience(
-        S, freqs=freqs, harmonics=harms, weights=weights, aggregate=np.ma.max, kind="quadratic", fill_value=0.0
+        S,
+        freqs=freqs,
+        harmonics=harms,
+        weights=weights,
+        aggregate=np.max,
+        kind="quadratic",
+        fill_value=0.0,
     )
 
     expected = np.array(
@@ -714,7 +826,7 @@ def test_magphase_zero():
     assert S.dtype is np.dtype("float32")
     assert P.dtype is np.dtype("complex64")
     assert np.allclose(S, 0)
-    assert np.allclose(P, 1+0j)
+    assert np.allclose(P, 1 + 0j)
 
 
 def test_magphase_denormalized():
@@ -725,7 +837,7 @@ def test_magphase_denormalized():
     assert S.dtype is np.dtype("float32")
     assert P.dtype is np.dtype("complex64")
     assert np.allclose(S, 1.0e-42)
-    assert np.allclose(P, 0+1j)
+    assert np.allclose(P, 0 + 1j)
 
 
 def test_magphase_real():
@@ -741,7 +853,7 @@ def test_magphase_real():
             [P[0, 0], P[0, 1] ** 2],  # negative zero can have phase +1 or -1
             [P[1, 0], P[1, 1]],
         ],
-        np.array([[-1+0j, 1+0j], [1+0j, 1+0j]])
+        np.array([[-1 + 0j, 1 + 0j], [1 + 0j, 1 + 0j]]),
     )
 
 
@@ -829,7 +941,7 @@ def test_get_duration_filename():
     filename = os.path.join("tests", "data", "test2_8000.wav")
     true_duration = 30.197625
 
-    duration_fn = librosa.get_duration(filename=filename)
+    duration_fn = librosa.get_duration(path=filename)
     y, sr = librosa.load(filename, sr=None)
     duration_y = librosa.get_duration(y=y, sr=sr)
 
@@ -841,7 +953,7 @@ def test_get_duration_mp3():
     filename = os.path.join("tests", "data", "test1_22050.mp3")
     true_duration = 4.587528344671202
 
-    duration_fn = librosa.get_duration(filename=filename)
+    duration_fn = librosa.get_duration(path=filename)
     y, sr = librosa.load(filename, sr=None)
     duration_y = librosa.get_duration(y=y, sr=sr)
     # mp3 duration at low sampling rate isn't too reliable
@@ -851,12 +963,12 @@ def test_get_duration_mp3():
 
 @pytest.mark.xfail(raises=librosa.ParameterError)
 def test_get_duration_fail():
-    librosa.get_duration(y=None, S=None, filename=None)
+    librosa.get_duration(y=None, S=None, path=None)
 
 
 @pytest.mark.parametrize(
     "y",
-    [np.random.randn(256, 256), np.exp(1.0j * np.random.randn(256, 256))],
+    [np.random.randn(256, 384), np.exp(1.0j * np.random.randn(256, 384))],
     ids=["real", "complex"],
 )
 @pytest.mark.parametrize("axis", [0, 1, -1])
@@ -940,7 +1052,7 @@ def test_to_mono(y):
 
 
 @pytest.mark.parametrize(
-    "y", [np.ones((2, 10)), np.ones((2, 3, 10)), np.ones((2,3,4,10))]
+    "y", [np.ones((2, 10)), np.ones((2, 3, 10)), np.ones((2, 3, 4, 10))]
 )
 def test_to_mono_multi(y):
     y_mono = librosa.to_mono(y)
@@ -950,7 +1062,7 @@ def test_to_mono_multi(y):
 
 
 @pytest.mark.parametrize("data", [np.random.randn(32)])
-@pytest.mark.parametrize("threshold", [None, 0, 1e-10])
+@pytest.mark.parametrize("threshold", [0, 1e-10])
 @pytest.mark.parametrize("ref_magnitude", [None, 0.1, np.max])
 @pytest.mark.parametrize("pad", [False, True])
 @pytest.mark.parametrize("zp", [False, True])
@@ -1042,7 +1154,7 @@ def test_piptrack_errors():
     pitches, mags = librosa.piptrack(
         y=None,
         sr=22050,
-        S=np.asarray([[1, 0, 0]]).T,
+        S=np.asarray([[1.0, 0.0, 0.0]]).T,
         n_fft=4096,
         hop_length=None,
         fmin=150,
@@ -1059,45 +1171,84 @@ def test_yin_tone(freq):
 
 
 def test_yin_chirp():
+    # test yin on a chirp, using output from the vamp plugin as ground truth
     y = librosa.chirp(fmin=220, fmax=640, duration=1.0)
-    f0 = librosa.yin(y, fmin=110, fmax=880, center=False)
+    f0 = librosa.yin(
+        y, fmin=110, fmax=880, center=False, frame_length=1024, hop_length=512
+    )
+
+    # adjust frames to the removal of win_length from yin
+    f0 = f0[:-2]
+
     target_f0 = np.load(os.path.join("tests", "data", "pitch-yin.npy"))
     assert np.allclose(np.log2(f0), np.log2(target_f0), rtol=0, atol=1e-2)
 
 
+def test_yin_chirp_instant():
+    # test yin on a chirp, using frame-wise instantaneous frequency as ground truth
+    sr = 22050
+    chirp_min, chirp_max = 220, 640
+
+    t = np.arange(sr) / sr
+    f = chirp_min * (chirp_max / chirp_min) ** t
+
+    fl = 2048
+    hl = 512
+
+    y = librosa.chirp(fmin=chirp_min, fmax=chirp_max, sr=sr, duration=1.0, linear=False)
+    target_f0 = librosa.util.frame(f, frame_length=fl, hop_length=hl).mean(axis=0)
+
+    f0 = librosa.yin(
+        y, fmin=110, fmax=880, sr=sr, frame_length=fl, hop_length=hl, center=False
+    )
+    assert np.allclose(np.log2(f0), np.log2(target_f0), rtol=0, atol=1e-2)
+
+
 @pytest.mark.xfail(raises=librosa.ParameterError)
 @pytest.mark.parametrize(
-    "fmin,fmax,win_length,frame_length",
+    "fmin,fmax,frame_length",
     [
-        (None, None, None, 2048),
-        (110, None, None, 2048),
-        (None, 880, None, 2048),
-        (110, 880, 2049, 2048),
+        (None, None, 2048),  # neither
+        (110, None, 2048),  # no fmax
+        (None, 880, 2048),  # no fmin
+        (-1, 440, 2048),  # Negative fmin
+        (440, 220, 2048),  # fmin > fmax
+        (440, 16000, 2048),  # fmax > nyquist
+        (10, 21, 2048),  # sr / fmin >= frame_length - 1
     ],
 )
-def test_yin_fail(fmin, fmax, win_length, frame_length):
+def test_yin_fail(fmin, fmax, frame_length):
     y = librosa.tone(110, duration=1.0)
-    librosa.yin(
-        y, fmin=fmin, fmax=fmax, win_length=win_length, frame_length=frame_length
-    )
+    librosa.yin(y, fmin=fmin, fmax=fmax, frame_length=frame_length)
+
+
+def test_yin_warn():
+    y = librosa.tone(110, duration=1.0)
+
+    # win_length is deprecated
+    with pytest.warns(FutureWarning, match="deprecated"):
+        librosa.yin(y, fmin=110, fmax=1000, win_length=1024)
+
+    # sr / fmin >= frame_length // 2
+    with pytest.warns(UserWarning, match="two periods"):
+        librosa.yin(y, fmin=20, fmax=1000)
 
 
 @pytest.mark.parametrize("freq", [110, 220, 440, 880])
 def test_pyin_tone(freq):
     y = librosa.tone(freq, duration=1.0)
     f0, _, _ = librosa.pyin(y, fmin=110, fmax=1000, center=False)
-    # Skip the first frame, since the voicing prior does not allow it
-    assert np.allclose(np.log2(f0[1:]), np.log2(freq), rtol=0, atol=1e-2)
+    assert np.allclose(np.log2(f0), np.log2(freq), rtol=0, atol=1e-2)
 
 
 def test_pyin_multi():
     y = np.stack([librosa.tone(440, duration=1.0), librosa.tone(560, duration=1.0)])
 
     # Taper the signal
-    h = librosa.filters.get_window('triangle', y.shape[-1])
+    h = librosa.filters.get_window("triangle", y.shape[-1])
 
     # Filter it
-    y = y * h[np.newaxis,:]
+    y = y * h[np.newaxis, :]
 
     # Disable nans so we can use allclose checks
     fall, vall, vpall = librosa.pyin(y, fmin=100, fmax=1000, center=False, fill_na=-1)
@@ -1112,32 +1263,55 @@ def test_pyin_multi():
     assert np.allclose(vpall[1], vp1)
 
 
+@pytest.mark.skipif(
+    sys.platform == "darwin", reason="Skip on OSX due to openblas issue"
+)
 def test_pyin_multi_center():
+    # Note: this test has issues on OSX with libopenblas 0.3.26,
+    # so we disable it for now.  We may re-enable it some time in the future.
     y = np.stack([librosa.tone(440, duration=1.0), librosa.tone(560, duration=1.0)])
 
     # Taper the signal
-    h = librosa.filters.get_window('triangle', y.shape[-1])
+    h = librosa.filters.get_window("triangle", y.shape[-1])
 
     # Filter it
-    y = y * h[np.newaxis,:]
+    y = y * h[np.newaxis, :]
 
-    # Disable nans so we can use allclose checks
-    fleft, vleft, vpleft = librosa.pyin(y, fmin=100, fmax=1000, center=False, fill_na=-1)
-    fc, vc, vpc = librosa.pyin(y, fmin=100, fmax=1000, center=True, fill_na=-1)
+    fleft, vleft, vpleft = librosa.pyin(y, fmin=100, fmax=1000, center=False)
+    fc, vc, vpc = librosa.pyin(y, fmin=100, fmax=1000, center=True)
 
     # Centering will pad by half a frame on either side
     # hop length is one quarter frame
     # ==> match on 2:-2
 
-    assert np.allclose(fleft, fc[..., 2:-2])
-    assert np.allclose(vleft, vc[..., 2:-2])
+    # Loosening tolerances here to account for platform differences
     assert np.allclose(vpleft, vpc[..., 2:-2])
+    assert np.allclose(vleft, vc[..., 2:-2])
+    assert np.allclose(fleft, fc[..., 2:-2], equal_nan=True), np.max(
+        np.abs(fleft - fc[..., 2:-2])
+    )
 
 
 def test_pyin_chirp():
+    # test yin on a chirp, using output from the vamp plugin as ground truth
     y = librosa.chirp(fmin=220, fmax=640, duration=1.0)
     y = np.pad(y, (22050,))
-    f0, voiced_flag, _ = librosa.pyin(y, fmin=110, fmax=880, center=False)
+
+    # default values as set in https://code.soundsoftware.ac.uk/projects/pyin/repository/
+    f0, voiced_flag, _ = librosa.pyin(
+        y,
+        fmin=60,
+        fmax=900,
+        center=False,
+        frame_length=1024,
+        hop_length=512,
+        resolution=0.2,
+    )
+
+    # adjust frames to the removal of win_length from yin
+    f0 = f0[:-2]
+    voiced_flag = voiced_flag[:-2]
+
     target_f0 = np.load(os.path.join("tests", "data", "pitch-pyin.npy"))
     # test if correct frames are voiced
     assert np.array_equal(voiced_flag, target_f0 > 0)
@@ -1147,21 +1321,66 @@ def test_pyin_chirp():
     )
 
 
+def test_pyin_chirp_instant():
+    # test pyin on a chirp, using frame-wise instantaneous frequency as ground truth
+    sr = 22050
+    chirp_min, chirp_max = 220, 640
+
+    t = np.arange(sr) / sr
+    f = chirp_min * (chirp_max / chirp_min) ** t
+    f = np.pad(f, (sr,))
+
+    fl = 2048
+    hl = 512
+
+    # Note: this raises warnings on the empty frames
+    target_f0 = librosa.util.frame(f, frame_length=fl, hop_length=hl)
+    with pytest.warns(RuntimeWarning):
+        target_f0 = target_f0.mean(axis=0, where=target_f0 > 0)
+
+    y = librosa.chirp(fmin=chirp_min, fmax=chirp_max, sr=sr, duration=1.0, linear=False)
+    y = np.pad(y, (sr,))
+
+    f0, voiced_flag, _ = librosa.pyin(
+        y, fmin=110, fmax=880, frame_length=fl, hop_length=hl, center=False
+    )
+
+    # test if correct frames are voiced
+    assert np.array_equal(voiced_flag, target_f0 > 0)
+
+    # test voiced frames are within one cent of the target
+    cents = np.log2(f0[voiced_flag])
+    target_cents = np.log2(target_f0[target_f0 > 0])
+
+    assert np.allclose(
+        np.log2(cents[1:-1]), np.log2(target_cents[1:-1]), rtol=0, atol=1e-2
+    )
+
+    # higher tolerance for the first and last frames, accounting for abrupt start / end
+    assert np.abs(cents[0] - target_cents[0]) <= 1e-1
+    assert np.abs(cents[-1] - target_cents[-1]) <= 1e-1
+
+
 @pytest.mark.xfail(raises=librosa.ParameterError)
 @pytest.mark.parametrize(
-    "fmin,fmax,win_length,frame_length",
+    "fmin,fmax,frame_length",
     [
-        (None, None, None, 2048),
-        (110, None, None, 2048),
-        (None, 880, None, 2048),
-        (110, 880, 2049, 2048),
+        (None, None, 2048),
+        (110, None, 2048),
+        (None, 880, 2048),
     ],
 )
-def test_pyin_fail(fmin, fmax, win_length, frame_length):
+def test_pyin_fail(fmin, fmax, frame_length):
     y = librosa.tone(110, duration=1.0)
-    librosa.pyin(
-        y, fmin=fmin, fmax=fmax, win_length=win_length, frame_length=frame_length
-    )
+    librosa.pyin(y, fmin=fmin, fmax=fmax, frame_length=frame_length)
+
+
+def test_pyin_warn():
+    y = librosa.tone(110, duration=1.0)
+
+    # win_length is deprecated
+    with pytest.warns(FutureWarning, match="deprecated"):
+        librosa.pyin(y, fmin=110, fmax=1000, win_length=1024)
 
 
 @pytest.mark.parametrize("freq", [110, 220, 440, 880])
@@ -1188,7 +1407,9 @@ def test_piptrack(freq, n_fft):
 @pytest.mark.parametrize("bins_per_octave", [12])
 @pytest.mark.parametrize("resolution", [1e-2])
 @pytest.mark.parametrize("sr", [11025, 22050])
-def test_estimate_tuning(sr, center_note, tuning, bins_per_octave, resolution):
+def test_estimate_tuning(
+    sr, center_note: int, tuning: float, bins_per_octave, resolution
+):
 
     target_hz = librosa.midi_to_hz(center_note + tuning)
 
@@ -1214,15 +1435,16 @@ def test_estimate_tuning(sr, center_note, tuning, bins_per_octave, resolution):
     assert max_dev <= 3 * resolution
 
 
-@pytest.mark.parametrize("y", [np.zeros(1000)])
+@pytest.mark.parametrize("y", [np.zeros(4000)])
 @pytest.mark.parametrize("sr", [11025, 22050])
 @pytest.mark.parametrize("resolution", [1e-2])
 @pytest.mark.parametrize("bins_per_octave", [12])
 def test_estimate_tuning_null(y, sr, resolution, bins_per_octave):
-    tuning_est = librosa.estimate_tuning(
-        resolution=resolution, bins_per_octave=bins_per_octave, y=y, sr=sr
-    )
-    assert np.allclose(tuning_est, 0)
+    with pytest.warns(UserWarning, match="Trying to estimate tuning"):
+        tuning_est = librosa.estimate_tuning(
+            resolution=resolution, bins_per_octave=bins_per_octave, y=y, sr=sr
+        )
+        assert np.allclose(tuning_est, 0)
 
 
 @pytest.mark.parametrize("n_fft", [1024, 755, 2048, 2049])
@@ -1265,10 +1487,15 @@ def test__spectrogram(y_22050, n_fft, hop_length, power):
     # And only the spectrogram but with incorrect n_fft
     S_, n_fft_ = librosa.core.spectrum._spectrogram(S=S, n_fft=2 * n_fft, power=power)
     assert np.allclose(S, S_)
-    
+
     assert np.allclose(2 * (S.shape[-2] - 1), n_fft_)
 
 
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test__spectrogram_no_nfft():
+    librosa.core.spectrum._spectrogram(S=None, n_fft=None)
+
+
 @pytest.mark.parametrize(
     "x", [np.linspace(0, 2e5, num=10), np.linspace(0, 2e5, num=10) * np.exp(1.0j)]
 )
@@ -1277,7 +1504,11 @@ def test__spectrogram(y_22050, n_fft, hop_length, power):
 @pytest.mark.parametrize("top_db", [None, 0, 40, 80])
 def test_power_to_db(x, ref, amin, top_db):
 
-    y = librosa.power_to_db(x, ref=ref, amin=amin, top_db=top_db)
+    if np.iscomplexobj(x):
+        with pytest.warns(UserWarning, match="power_to_db was called on complex input"):
+            y = librosa.power_to_db(x, ref=ref, amin=amin, top_db=top_db)
+    else:
+        y = librosa.power_to_db(x, ref=ref, amin=amin, top_db=top_db)
 
     assert np.isrealobj(y)
     assert y.shape == x.shape
@@ -1298,8 +1529,8 @@ def test_power_to_db_fail(x, top_db, amin):
 def test_power_to_db_inv(erp, k):
 
     y_true = (k - erp) * 10
-    x = 10.0 ** k
-    rp = 10.0 ** erp
+    x = 10.0**k
+    rp = 10.0**erp
     y = librosa.power_to_db(x, ref=rp, top_db=None)
 
     assert np.isclose(y, y_true)
@@ -1314,7 +1545,7 @@ def test_amplitude_to_db():
     x = np.abs(np.random.randn(1000)) + NOISE_FLOOR
 
     db1 = librosa.amplitude_to_db(x, top_db=None)
-    db2 = librosa.power_to_db(x ** 2, top_db=None)
+    db2 = librosa.power_to_db(x**2, top_db=None)
 
     assert np.allclose(db1, db2)
 
@@ -1327,21 +1558,39 @@ def test_amplitude_to_db_complex():
     # Make some noise
     x = np.abs(np.random.randn(1000)) + NOISE_FLOOR
 
-    with warnings.catch_warnings(record=True) as out:
-        db1 = librosa.amplitude_to_db(x.astype(np.complex), top_db=None)
-        assert len(out) > 0
-        assert "complex" in str(out[0].message).lower()
+    with pytest.warns(UserWarning, match="amplitude_to_db was called on complex input"):
+        db1 = librosa.amplitude_to_db(x.astype(complex), top_db=None)
 
-    db2 = librosa.power_to_db(x ** 2, top_db=None)
+    db2 = librosa.power_to_db(x**2, top_db=None)
 
     assert np.allclose(db1, db2)
 
 
+def test_amplitude_to_db_scalar():
+    assert np.isclose(librosa.amplitude_to_db(1), 0)
+    assert np.isclose(librosa.amplitude_to_db(2), 6.0206)
+
+
+def test_power_to_db_scalar():
+    assert np.isclose(librosa.power_to_db(1), 0)
+    assert np.isclose(librosa.power_to_db(2), 3.0103)
+
+
+def test_db_to_amplitude_scalar():
+    assert np.isclose(librosa.db_to_amplitude(0), 1)
+    assert np.isclose(librosa.db_to_amplitude(6.0206), 2)
+
+
+def test_db_to_power_scalar():
+    assert np.isclose(librosa.db_to_power(0), 1)
+    assert np.isclose(librosa.db_to_power(3.0103), 2)
+
+
 @pytest.mark.parametrize("ref_p", range(-3, 4))
 @pytest.mark.parametrize("xp", [(np.abs(np.random.randn(1000)) + 1e-5) ** 2])
 def test_db_to_power_inv(ref_p, xp):
 
-    ref = 10.0 ** ref_p
+    ref = 10.0**ref_p
     db = librosa.power_to_db(xp, ref=ref, top_db=None)
     xp2 = librosa.db_to_power(db, ref=ref)
 
@@ -1353,8 +1602,8 @@ def test_db_to_power_inv(ref_p, xp):
 def test_db_to_power(erp, db):
 
     y = db
-    rp = 10.0 ** erp
-    x_true = 10.0 ** erp * (10.0 ** (0.1 * db))
+    rp = 10.0**erp
+    x_true = 10.0**erp * (10.0 ** (0.1 * db))
 
     x = librosa.db_to_power(y, ref=rp)
 
@@ -1362,10 +1611,10 @@ def test_db_to_power(erp, db):
 
 
 @pytest.mark.parametrize("ref_p", range(-3, 4))
-@pytest.mark.parametrize("xp", [(np.abs(np.random.randn(1000)) + 1e-5)])
+@pytest.mark.parametrize("xp", [np.abs(np.random.randn(1000)) + 1e-5])
 def test_db_to_amplitude_inv(xp, ref_p):
 
-    ref = 10.0 ** ref_p
+    ref = 10.0**ref_p
     db = librosa.amplitude_to_db(xp, ref=ref, top_db=None)
     xp2 = librosa.db_to_amplitude(db, ref=ref)
 
@@ -1418,6 +1667,7 @@ def test_clicks(
     if times is not None:
         nmax = librosa.time_to_samples(times, sr=sr).max()
     else:
+        assert frames is not None
         nmax = librosa.frames_to_samples(frames, hop_length=hop_length).max()
 
     if length is not None:
@@ -1464,7 +1714,7 @@ def test_tone(frequency, sr, length, duration, phi):
     if length is not None:
         assert len(y) == length
     else:
-        assert len(y) == np.ceil(duration * sr)
+        assert len(y) == int(duration * sr)
 
 
 @pytest.mark.xfail(raises=librosa.ParameterError)
@@ -1495,7 +1745,7 @@ def test_chirp(fmin, fmax, sr, length, duration, linear, phi):
     if length is not None:
         assert len(y) == length
     else:
-        assert len(y) == np.ceil(duration * sr)
+        assert len(y) == int(duration * sr)
 
 
 @pytest.mark.xfail(raises=librosa.ParameterError)
@@ -1541,7 +1791,7 @@ def y_orig():
 def test_fmt_scale(y_orig, y_res, n_fmt, kind, atol, SCALE, OVER_SAMPLE):
 
     # Make sure our signals preserve energy
-    assert np.allclose(np.sum(y_orig ** 2), np.sum(y_res ** 2))
+    assert np.allclose(np.sum(y_orig**2), np.sum(y_res**2))
 
     # Scale-transform the original
     f_orig = librosa.fmt(
@@ -1617,7 +1867,7 @@ def test_harmonics_1d():
             assert np.allclose(vals, y[: len(vals)])
         else:
             # Else check that harmonics match
-            step = h[i]
+            step = cast(int, h[i])
             vals = y[::step]
             assert np.allclose(vals, yh[i, : len(vals)])
 
@@ -1641,13 +1891,13 @@ def test_harmonics_2d():
             assert np.allclose(vals, y[: len(vals)])
         else:
             # Else check that harmonics match
-            step = h[i]
+            step = cast(int, h[i])
             vals = y[::step]
             assert np.allclose(vals, yh[i, : len(vals)])
 
 
 def test_harmonics_1d_nonunique():
-    x = np.arange(-8, 8)**2
+    x = np.arange(-8, 8) ** 2
     y = np.linspace(-8, 8, num=len(x), endpoint=False) ** 2
 
     h = [0.25, 0.5, 1, 2, 4]
@@ -1690,7 +1940,7 @@ def test_harmonics_2d_varying():
             assert np.allclose(vals, y[: len(vals)])
         else:
             # Else check that harmonics match
-            step = h[i]
+            step = cast(int, h[i])
             vals = y[::step]
             assert np.allclose(vals, yh[i, : len(vals)])
 
@@ -1753,15 +2003,15 @@ def test_iirt_peaks():
     sample_rates = np.asarray(
         len(np.arange(40, 46))
         * [
-            1000,
+            1000.0,
         ]
         + len(np.arange(46, 80))
         * [
-            1750,
+            1750.0,
         ]
         + len(np.arange(80, 95))
         * [
-            4000,
+            4000.0,
         ]
     )
 
@@ -1806,15 +2056,15 @@ def S_pcen():
 @pytest.mark.parametrize(
     "gain,bias,power,b,time_constant,eps,ms",
     [
-        (-1, 1, 1, 0.5, 0.5, 1e-6, 1),  #   gain < 0
-        (1, -1, 1, 0.5, 0.5, 1e-6, 1),  #   bias < 0
-        (1, 1, -0.1, 0.5, 0.5, 1e-6, 1),  #   power < 0
-        (1, 1, 1, -2, 0.5, 1e-6, 1),  #   b < 0
-        (1, 1, 1, 2, 0.5, 1e-6, 1),  #   b > 1
-        (1, 1, 1, 0.5, -2, 1e-6, 1),  #   time_constant <= 0
-        (1, 1, 1, 0.5, 0.5, 0, 1),  #   eps <= 0
+        (-1, 1, 1, 0.5, 0.5, 1e-6, 1),  # gain < 0
+        (1, -1, 1, 0.5, 0.5, 1e-6, 1),  # bias < 0
+        (1, 1, -0.1, 0.5, 0.5, 1e-6, 1),  # power < 0
+        (1, 1, 1, -2, 0.5, 1e-6, 1),  # b < 0
+        (1, 1, 1, 2, 0.5, 1e-6, 1),  # b > 1
+        (1, 1, 1, 0.5, -2, 1e-6, 1),  # time_constant <= 0
+        (1, 1, 1, 0.5, 0.5, 0, 1),  # eps <= 0
         (1, 1, 1, 0.5, 0.5, 1e-6, 1.5),
-        (1, 1, 1, 0.5, 0.5, 1e-6, 0),  #   max_size not int, < 1
+        (1, 1, 1, 0.5, 0.5, 1e-6, 0),  # max_size not int, < 1
     ],
 )
 def test_pcen_failures(gain, bias, power, b, time_constant, eps, ms, S_pcen):
@@ -1837,7 +2087,7 @@ def test_pcen_power(S_pcen, p):
     P = librosa.pcen(
         S_pcen, gain=0, bias=0, power=p, b=1, time_constant=0.5, eps=1e-6, max_size=1
     )
-    assert np.allclose(P, S_pcen ** p)
+    assert np.allclose(P, S_pcen**p)
 
 
 def test_pcen_ones(S_pcen):
@@ -1864,7 +2114,7 @@ def test_pcen_drc(S_pcen, bias, power):
 
 
 def test_pcen_complex():
-    S = np.ones((9, 30), dtype=np.complex)
+    S = np.ones((9, 30), dtype=complex)
     Pexp = np.ones((9, 30))
 
     with warnings.catch_warnings(record=True) as out:
@@ -2010,6 +2260,8 @@ def test_pcen_stream_multi(axis):
     slice2 = [slice(None)] * x.ndim
     slice2[axis] = slice(10, None)
 
+    slice1 = tuple(slice1)
+    slice2 = tuple(slice2)
     # Compute pcen piecewise
     p1, zf1 = librosa.pcen(x[slice1], return_zf=True, axis=axis)
     p2, zf2 = librosa.pcen(x[slice2], zi=zf1, return_zf=True, axis=axis)
@@ -2022,28 +2274,31 @@ def test_pcen_stream_multi(axis):
 
 
 def test_get_fftlib():
-    import numpy.fft as fft
+    import scipy.fft as fft
 
     assert librosa.get_fftlib() is fft
 
 
 def test_set_fftlib():
-    librosa.set_fftlib("foo")
-    assert librosa.get_fftlib() == "foo"
-    librosa.set_fftlib()
+    with pytest.warns(FutureWarning):
+        librosa.set_fftlib("foo")  # type: ignore
+    assert librosa.get_fftlib() == "foo"  # type: ignore
+    with pytest.warns(FutureWarning):
+        librosa.set_fftlib()
 
 
 def test_reset_fftlib():
-    import numpy.fft as fft
+    import scipy.fft as fft
 
-    librosa.set_fftlib()
+    with pytest.warns(FutureWarning):
+        librosa.set_fftlib()
     assert librosa.get_fftlib() is fft
 
 
 @pytest.fixture
 def y_chirp():
     sr = 22050
-    y = librosa.chirp(fmin=55, fmax=55 * 2 ** 7, length=sr // 8, sr=sr)
+    y = librosa.chirp(fmin=55, fmax=55 * 2**7, length=sr // 8, sr=sr)
     return y
 
 
@@ -2085,7 +2340,6 @@ def test_griffinlim(
         n_fft=n_fft,
         window=window,
         center=center,
-        dtype=dtype,
         pad_mode=pad_mode,
     )
 
@@ -2121,6 +2375,12 @@ def test_griffinlim_badinit():
     librosa.griffinlim(x, init="garbage")
 
 
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_griffinlim_badrng():
+    x = np.zeros((33, 3))
+    librosa.griffinlim(x, random_state="garbage")  # type: ignore
+
+
 @pytest.mark.xfail(raises=librosa.ParameterError)
 def test_griffinlim_momentum():
     x = np.zeros((33, 3))
@@ -2240,7 +2500,9 @@ def test_stream(
         # frame this for easy checking
         y_b_mono = librosa.to_mono(y_block)
         if len(y_b_mono) >= frame_length:
-            y_b_frame = librosa.util.frame(y_b_mono, frame_length=frame_length, hop_length=hop_length)
+            y_b_frame = librosa.util.frame(
+                y_b_mono, frame_length=frame_length, hop_length=hop_length
+            )
             y_frame_stream.append(y_b_frame)
 
     # Concatenate the framed blocks together
@@ -2257,7 +2519,9 @@ def test_stream(
         path, sr=None, dtype=dtype, mono=True, offset=offset, duration=duration
     )
     # First, check the rate
-    y_frame = librosa.util.frame(y_full, frame_length=frame_length, hop_length=hop_length)
+    y_frame = librosa.util.frame(
+        y_full, frame_length=frame_length, hop_length=hop_length
+    )
 
     # Raw audio will not be padded
     n = y_frame.shape[1]
@@ -2334,3 +2598,169 @@ def test_mu_expand_badmu():
 )
 def test_mu_expand_badx(x):
     librosa.mu_expand(x, quantize=False)
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_stft_bad_prealloc_shape():
+    y = np.zeros(22050)
+
+    # Output shape here is incorrect, and should trigger a failure
+    S1 = librosa.stft(
+        y, n_fft=512, hop_length=128, out=np.zeros((100, 10), dtype=complex)
+    )
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_stft_bad_prealloc_dtype():
+    y = np.zeros(22050)
+    D = librosa.stft(y)
+
+    Dbad = np.zeros(D.shape, dtype=np.float32)
+    librosa.stft(y, out=Dbad)
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_istft_bad_prealloc_shape():
+    D = np.zeros((1025, 5), dtype=np.complex64)
+    librosa.istft(D, out=np.zeros(100))
+
+
+# Tests to force audioread decoding
+def test_load_force_audioread():
+    path = os.path.join("tests", "data", "test2_8000.mkv")
+    with warnings.catch_warnings(record=True) as out:
+        y, sr = librosa.load(path)
+
+        assert len(out) > 0
+        assert "audioread" in str(out[0].message).lower()
+
+
+@pytest.mark.filterwarnings("ignore:PySoundFile failed")
+def test_get_duration_audioread():
+    path = os.path.join("tests", "data", "test2_8000.mkv")
+    duration = librosa.get_duration(path=path)
+
+    # Duration is 30.2 seconds if using ffmpeg
+    # Duration is 30.23 seconds otherwise (eg gstreamer)
+    # To avoid floating point issues, we'll just check that it's close
+    assert np.isclose(duration, 30.2, atol=0.1)
+
+
+@pytest.mark.filterwarnings("ignore:PySoundFile failed")
+def test_get_samplerate_audioread():
+    path = os.path.join("tests", "data", "test2_8000.mkv")
+    sr = librosa.get_samplerate(path=path)
+
+    assert sr == 8000
+
+
+def test_f0_harmonics_static():
+
+    freqs = np.arange(8)
+
+    data = np.multiply.outer(freqs, np.arange(2, 5))
+    # array([[ 0,  0,  0],
+    #        [ 2,  3,  4],
+    #        [ 4,  6,  8],
+    #        [ 6,  9, 12],
+    #        [ 8, 12, 16],
+    #        [10, 15, 20],
+    #        [12, 18, 24],
+    #        [14, 21, 28]])
+
+    f0 = np.array([1, 2, 0])
+    harmonics = [0.5, 1, 3]
+
+    yh = librosa.f0_harmonics(data, f0=f0, freqs=freqs, harmonics=harmonics)
+
+    assert yh.shape[0] == len(harmonics)
+    assert yh.shape[1:] == data.shape[1:]
+
+    # The 1 here comes from linear interpolation of the 0.5 subharmonic
+    # All else are data[1,2,3]
+    assert np.allclose(yh[:, 0], [1, 2, 6])
+    # Values here come from f0 = 2
+    assert np.allclose(yh[:, 1], [3, 6, 18])
+    # Last frame has f0 = 0, so all harmonics will evaluate to 0
+    assert np.allclose(yh[:, 2], [0, 0, 0])
+
+
+def test_f0_harmonics_dynamic():
+
+    # Cook up a dynamic frequency grid
+    freqs = np.add.outer(np.arange(8), np.arange(3))
+    # array([[0, 1, 2],
+    #        [1, 2, 3],
+    #        [2, 3, 4],
+    #        [3, 4, 5],
+    #        [4, 5, 6],
+    #        [5, 6, 7],
+    #        [6, 7, 8],
+    #        [7, 8, 9]])
+
+    # Broadcast multiply to make some measurements
+    data = freqs * np.arange(2, 5)
+    # array([[ 0,  3,  8],
+    #        [ 2,  6, 12],
+    #        [ 4,  9, 16],
+    #        [ 6, 12, 20],
+    #        [ 8, 15, 24],
+    #        [10, 18, 28],
+    #        [12, 21, 32],
+    #        [14, 24, 36]])
+
+    # f0 at each frame
+    f0 = np.array([2, 4, 5])
+
+    # Harmonics
+    harmonics = [0.5, 1, 2]
+
+    # Interpolate
+    yh = librosa.f0_harmonics(data, f0=f0, freqs=freqs, harmonics=harmonics)
+
+    assert yh.shape[0] == len(harmonics)
+    assert yh.shape[1:] == data.shape[1:]
+
+    # f0 in frame 0 = 2, f0 * h => [1, 2, 4]
+    # data[f0 * h] = [2, 4, 8]
+    assert np.allclose(yh[:, 0], [2, 4, 8])
+    # f0 in frame 1 = 4, f0 * h => [2, 4, 8]
+    # data[f0 * h] = [6, 12, 24]
+    assert np.allclose(yh[:, 1], [6, 12, 24])
+    # f0 in frame 2 = 5, f0 * h = [2.5, 5, 10]
+    # interpolation happens here
+    # last frequency falls off the edge of the frequency grid, filled as 0
+    assert np.allclose(yh[:, 2], [10, 20, 0])
+
+
+def test_f0_harmonics_1d_nonunique():
+    freqs = np.arange(-8, 8) ** 2
+    data = np.multiply.outer(freqs, np.arange(5))
+
+    h = [1, 2, 3]
+    f0 = np.ones(data.shape[-1])
+    with pytest.warns(UserWarning):
+        librosa.f0_harmonics(data, freqs=freqs, harmonics=h, f0=f0)
+
+
+def test_f0_harmonics_2d_nonunique():
+    freqs = np.add.outer(np.arange(-8, 8) ** 2, np.arange(5))
+
+    data = freqs * np.arange(5)
+
+    h = [1, 2, 3]
+    f0 = np.ones(data.shape[-1])
+    with pytest.warns(UserWarning):
+        librosa.f0_harmonics(data, freqs=freqs, harmonics=h, f0=f0)
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_f0_harmonics_incompat():
+
+    # Freq axis does not match data shape
+    freqs = np.arange(5)
+    data = np.zeros((6, 7))
+    f0 = np.arange(7)
+    harmonics = np.arange(1, 3)
+
+    librosa.f0_harmonics(data, freqs=freqs, harmonics=harmonics, f0=f0)
